@@ -1,33 +1,36 @@
+# pylint: disable=too-many-lines
+# pylint: disable=too-many-locals
+# pylint: disable=too-many-branches
+# pylint: disable=bare-except
+# pylint: disable=line-too-long
 '''
     Contains the general methods for manipulating directories.
 '''
+
 # import json
-import utils.resources
-import ftputil
-from ftputil.error import FTPOSError
-from re import search
+# from re import search
 import time
 import json
 import traceback
 import shutil
 import os
 from pathlib import Path
+from threading import Thread
+import logging
+# import utils.string_utils as strUtils
+import ftputil
+from ftputil.error import FTPOSError
+import colemen_string_utils as strUtils
 import utils.objectUtils as obj
 import utils.file_write as write
-import utils.string_utils as strUtils
-import colemen_string_utils as csu
 import utils.file as f
-import logging
-from threading import Thread
+import utils.resources
 
 logger = logging.getLogger(__name__)
 _THREAD_GET_FILES_ARRAY = []
-# todo - DOCUMENTATION FOR METHODS
-# pylint: disable=too-many-locals
-# pylint: disable=too-many-branches
 
 
-def create(path, dir_name=False, ftp=None):
+def create(path, dir_name=False, **kwargs):
     '''
         Create a directory or path of directories on the local machine or an FTP server.
 
@@ -39,6 +42,9 @@ def create(path, dir_name=False, ftp=None):
             The path to create or a path to where it should create the dir_name directory
         [`dir_name`=False] {str}
             The name of the directory to create.
+
+        Keyword Arguments
+        -----------------
         [`ftp`=None] {obj}
             A reference to the ftputil object.
 
@@ -55,6 +61,7 @@ def create(path, dir_name=False, ftp=None):
         `method_name`: create
     '''
     success = False
+    ftp = obj.get_kwarg(["ftp"], None, None, **kwargs)
     if dir_name is not False:
         path = os.path.join(path, dir_name)
 
@@ -65,18 +72,18 @@ def create(path, dir_name=False, ftp=None):
                 success = True
         except ftputil.error.PermanentError as error:
             print(f"error: {str(error)}")
-
-    if exists(path) is False:
-        Path(path).mkdir(parents=True, exist_ok=True)
-        if exists(path) is True:
-            success = True
     else:
-        success = True
+        if exists(path) is False:
+            Path(path).mkdir(parents=True, exist_ok=True)
+            if exists(path) is True:
+                success = True
+        else:
+            success = True
 
     return success
 
 
-def exists(file_path, ftp=None):
+def exists(file_path, **kwargs):
     '''
         Confirms that the directory file_path exists
 
@@ -87,6 +94,8 @@ def exists(file_path, ftp=None):
         `file_path` {str}
             The path to confirm.
 
+        Keyword Arguments
+        -----------------
         [`ftp`=None] {obj}
             A reference to the ftputil object.
 
@@ -103,12 +112,13 @@ def exists(file_path, ftp=None):
         `method_name`: exists
     '''
     dir_exists = False
-
-    if os.path.isdir(file_path) is True:
-        dir_exists = True
+    ftp = obj.get_kwarg(["ftp"], None, None, **kwargs)
 
     if ftp is not None:
         dir_exists = exists_ftp(file_path, ftp)
+    else:
+        if os.path.isdir(file_path) is True:
+            dir_exists = True
 
     return dir_exists
 
@@ -237,10 +247,10 @@ def get_folders(search_path=False, **kwargs):
                 dir_data['file_path'] = os.path.join(root, current_dir)
                 ignore = False
                 if len(exclude) > 0:
-                    if array_in_string(exclude, dir_data['file_path']) is True:
+                    if strUtils.parse.array_in_string(exclude, dir_data['file_path']) is True:
                         continue
                 if len(include) > 0:
-                    if array_in_string(include, dir_data['file_path']) is False:
+                    if strUtils.parse.array_in_string(include, dir_data['file_path']) is False:
                         continue
                 # if ignore_array is not False:
                 #     for x in ignore_array:
@@ -266,7 +276,8 @@ def get_folders_ftp(search_path=False, **kwargs):
         -------------------------
         `search_path` {str|list}
             The search path or list of paths to iterate.\n
-            This is the same as the keyword argument search_path, the kwarg is provided for consistency.
+            This is the same as the keyword argument search_path,
+            the kwarg is provided for consistency.
 
         Keyword Arguments
         -----------------
@@ -347,10 +358,10 @@ def get_folders_ftp(search_path=False, **kwargs):
                 dir_data['file_path'] = os.path.join(root, current_dir)
                 ignore = False
                 if len(exclude) > 0:
-                    if array_in_string(exclude, dir_data['file_path']) is True:
+                    if strUtils.parse.array_in_string(exclude, dir_data['file_path']) is True:
                         continue
                 if len(include) > 0:
-                    if array_in_string(include, dir_data['file_path']) is False:
+                    if strUtils.parse.array_in_string(include, dir_data['file_path']) is False:
                         continue
 
                 dir_array.append(dir_data)
@@ -381,20 +392,25 @@ def get_files(search_path=False, **kwargs):
                 An extension or list of extensions that the file must have.
 
             [`threaded`=True] {bool}
-                if True, the process is automatically multi-threaded, this makes indexing much faster,
+                if True, the process is multi-threaded,
+                this makes indexing much faster,
                 but it can easily overwhelm a cpu depending upon the drive.
 
             `data_include` {str|list}
                 The data to get for each file, the shorter this list the faster it will complete.
                 By default it will get this:
-                    ['file_name', 'extension', 'name_no_ext', 'dir_path', 'access_time', 'modified_time', 'created_time', 'size']
-                so the list you provide will limit the amount of reading/formatting needed to gather data.
+                    ['file_name', 'extension', 'name_no_ext', 'dir_path', 'access_time',
+                    'modified_time', 'created_time', 'size']
+                so the list you provide will limit the amount of
+                reading/formatting needed to gather data.
                 Example:
                     ['modified_time','size']  will take ~0.000129939 seconds per file on an SSD
 
-                    ['file_name', 'extension', 'name_no_ext', 'dir_path', 'access_time', 'modified_time', 'created_time', 'size']  
+                    ['file_name', 'extension', 'name_no_ext', 'dir_path', 'access_time',
+                    'modified_time', 'created_time', 'size']
 
-                    will take ~0.000173600 seconds per file on an SSD, it's not much but little things matter.. that's what she said.
+                    will take ~0.000173600 seconds per file on an SSD,
+                    it's not much but little things matter.. that's what she said.
 
         return
         ----------
@@ -423,13 +439,15 @@ def get_files(search_path=False, **kwargs):
     if isinstance(include, (str)):
         include = [include]
 
-    extension_array = strUtils.format_extension(obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
+    extension_array = strUtils.format.extension(
+        obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
     if isinstance(extension_array, (str)):
         extension_array = [extension_array]
 
     if threaded is True:
         for path in search_path:
-            gft = GetFilesThreaded(path, recursive=recursive, data_include=data_include, exclude=exclude, include=include, extensions=extension_array)
+            gft = GetFilesThreaded(path, recursive=recursive, data_include=data_include,
+                                   exclude=exclude, include=include, extensions=extension_array)
             return gft.master()
     # print(json.dumps(extension_array, indent=4))
     for path in search_path:
@@ -441,15 +459,15 @@ def get_files(search_path=False, **kwargs):
                     # ignore = False
                     # print(f"file_data['extension']: {file_data['extension']}")
                     if len(exclude) > 0:
-                        if array_in_string(exclude, file_data['file_path']) is True:
+                        if strUtils.parse.array_in_string(exclude, file_data['file_path']) is True:
                             continue
 
                     if len(include) > 0:
-                        if array_in_string(include, file_data['file_path']) is False:
+                        if strUtils.parse.array_in_string(include, file_data['file_path']) is False:
                             continue
 
                     if len(extension_array) > 0:
-                        file_ext = strUtils.format_extension(file_data['extension'])
+                        file_ext = strUtils.format.extension(file_data['extension'])
                         if file_ext not in extension_array:
                             continue
 
@@ -471,6 +489,11 @@ def get_files(search_path=False, **kwargs):
 
 
 class GetFilesThreaded:
+    '''
+        A class implementation of the get_files method.
+        This allows it use threading.
+    '''
+
     def __init__(self, search_path, **kwargs):
         self.threads = []
         self.file_array = []
@@ -478,7 +501,8 @@ class GetFilesThreaded:
         self.search_path = search_path
 
         if search_path is False:
-            self.search_path = obj.get_kwarg(['search path', 'search'], os.getcwd(), (str, list), **kwargs)
+            self.search_path = obj.get_kwarg(
+                ['search path', 'search'], os.getcwd(), (str, list), **kwargs)
         if isinstance(self.search_path, list) is False:
             self.search_path = [self.search_path]
 
@@ -488,8 +512,10 @@ class GetFilesThreaded:
         if isinstance(self.data_include, (str)):
             self.data_include = [self.data_include]
 
-        # ignore_array = obj.get_kwarg(['ignore', 'ignore_array', 'exclude'], [], (str, list), **kwargs)
-        self.exclude = obj.get_kwarg(['exclude', 'ignore', 'ignore array'], [], (list, str), **kwargs)
+        # ignore_array = obj.get_kwarg(
+            # ['ignore', 'ignore_array', 'exclude'], [], (str, list), **kwargs)
+        self.exclude = obj.get_kwarg(
+            ['exclude', 'ignore', 'ignore array'], [], (list, str), **kwargs)
         if isinstance(self.exclude, (str)):
             self.exclude = [self.exclude]
 
@@ -497,11 +523,30 @@ class GetFilesThreaded:
         if isinstance(self.include, (str)):
             self.include = [self.include]
 
-        self.extension_array = strUtils.format_extension(obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
+        self.extension_array = strUtils.format.extension(
+            obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
         if isinstance(self.extension_array, (str)):
             self.extension_array = [self.extension_array]
 
     def remove_thread_by_id(self, thread_id):
+        '''
+            Removes an active thread from self.threads
+
+            ----------
+
+            Arguments
+            -------------------------
+            `thread_id` {str}
+                The id of the thread to remove
+
+            Meta
+            ----------
+            `author`: Colemen Atwood
+            `created`: 12-19-2021 13:48:58
+            `memberOf`: dir
+            `version`: 1.0
+            `method_name`: remove_thread_by_id
+        '''
         threads = self.threads
         new_threads = []
         for thread in threads:
@@ -515,27 +560,63 @@ class GetFilesThreaded:
             # ignore = False
             # print(f"file_data['extension']: {file_data['extension']}")
             if len(self.exclude) > 0:
-                if array_in_string(self.exclude, file_data['file_path']) is True:
+                if strUtils.parse.array_in_string(self.exclude, file_data['file_path']) is True:
                     return
 
             if len(self.include) > 0:
-                if array_in_string(self.include, file_data['file_path']) is False:
+                if strUtils.parse.array_in_string(self.include, file_data['file_path']) is False:
                     return
 
             if len(self.extension_array) > 0:
-                file_ext = strUtils.format_extension(file_data['extension'])
+                file_ext = strUtils.format.extension(file_data['extension'])
                 if file_ext not in self.extension_array:
                     return
 
         self.file_array.append(file_data)
 
     def single_file_thread(self, data):
+        '''
+            Executes the get_data function on an array of files in a separate thread
+            and removes itself from self.threads once completed.
+
+            ----------
+
+            Arguments
+            -------------------------
+            `data` {dict}
+                a dictionary containing the file_paths and thread_id keys
+
+            Meta
+            ----------
+            `author`: Colemen Atwood
+            `created`: 12-19-2021 13:49:59
+            `memberOf`: dir
+            `version`: 1.0
+            `method_name`: single_file_thread
+        '''
         file_paths = data['file_paths']
         for file_path in file_paths:
             self._get_data_thread(file_path)
         self.remove_thread_by_id(data['thread_id'])
 
     def master(self):
+        '''
+            Executes the get files process using threads
+
+            ----------
+
+            Return {list}
+            ----------------------
+            A list of files found in the search_path.
+
+            Meta
+            ----------
+            `author`: Colemen Atwood
+            `created`: 12-19-2021 13:52:24
+            `memberOf`: dir
+            `version`: 1.0
+            `method_name`: master
+        '''
         for path in self.search_path:
             # pylint: disable=unused-variable
             for root, folders, files in os.walk(path):
@@ -546,7 +627,7 @@ class GetFilesThreaded:
 
                 file_paths = [os.path.join(root, x) for x in files]
                 data = {
-                    "thread_id": strUtils.generate_hash(json.dumps(file_paths)),
+                    "thread_id": strUtils.gen.hash(json.dumps(file_paths)),
                     "file_paths": file_paths
                 }
                 thread = Thread(target=self.single_file_thread, args=(data,))
@@ -644,7 +725,7 @@ class GetFilesThreaded:
         #     if isinstance(include, (str)):
         #         include = [include]
 
-        #     extension_array = strUtils.format_extension(obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
+        #     extension_array = strUtils.format.extension(obj.get_kwarg(['extensions', 'ext', 'extension'], [], (str, list), **kwargs))
         #     if isinstance(extension_array, (str)):
         #         extension_array = [extension_array]
 
@@ -811,19 +892,6 @@ def delete_ftp(file_path, ftp):
     return success
 
 
-def array_in_string(array, value, default=False):
-    if len(array) == 0:
-        return default
-    if isinstance(value, (str)) is False:
-        logger.warning('Second argument of array_in_string, must be a string.')
-        logger.warning(value)
-        return default
-    for item in array:
-        if item in value:
-            return True
-    return default
-
-
 def copy(src, dst=False, **kwargs):
     '''
         Copy a directory to another location.
@@ -842,7 +910,7 @@ def copy(src, dst=False, **kwargs):
         Keyword Arguments
         -------------------------
         `ftp` {obj}
-            A reference to the ftputil object. 
+            A reference to the ftputil object.
         [`ftp_direction`='local_to_server'] {str}
             The direction of the copying:
                 local_to_server: Copy local directories/files to the FTP server.
@@ -925,7 +993,7 @@ def mirror(src, dst, **kwargs):
         [`ftp_direction`='local_to_server'] {str}
             The direction of the copying:
                 local_to_server: Copy local directories/files to the FTP server.
-                server_to_local: Copy FTP server directories/files to the Local machine.            
+                server_to_local: Copy FTP server directories/files to the Local machine.
 
 
         Return {type}
@@ -1010,7 +1078,7 @@ def mirror_to_server(src, dst, **kwargs):
         Keyword Arguments
         -------------------------
         `ftp` {obj}
-            A reference to the ftputil object.        
+            A reference to the ftputil object.
         [`dirs_only`=False] {bool}
             If True, only directories are copied.
         [`recursive`=True] {bool}
@@ -1076,7 +1144,7 @@ def mirror_to_server(src, dst, **kwargs):
         logger.warning("Source path must exist.\nsource: %s", src)
         return False
 
-    if exists(dst, ftp) is False:
+    if exists(dst, ftp=ftp) is False:
         create(dst, ftp=ftp)
 
     dirs = get_folders(search_path=src, recursive=recursive,
@@ -1109,18 +1177,18 @@ def _parse_copy_data_from_obj(file_obj):
             data['src_path'] = file_obj[0]
             data['dst_path'] = file_obj[1]
         else:
-            print(f"Invalid list/tuple provided for copy file. Must be [source_file_path, destination_file_path]")
+            print("Invalid list/tuple provided for copy file. Must be [source_file_path, destination_file_path]")
     if isinstance(file_obj, (dict)):
         for syn in utils.resources.SRC_PATH_SYNONYMS:
-            synvar = obj._gen_variations(syn)
-            for sv in synvar:
-                if sv in file_obj:
-                    data['src_path'] = file_obj[sv]
+            synvar = obj.gen_variations(syn)
+            for synonym_variant in synvar:
+                if synonym_variant in file_obj:
+                    data['src_path'] = file_obj[synonym_variant]
         for syn in utils.resources.DEST_PATH_SYNONYMS:
-            synvar = obj._gen_variations(syn)
-            for sv in synvar:
-                if sv in file_obj:
-                    data['dst_path'] = file_obj[sv]
+            synvar = obj.gen_variations(syn)
+            for synonym_variant in synvar:
+                if synonym_variant in file_obj:
+                    data['dst_path'] = file_obj[synonym_variant]
 
     if exists(data['src_path']) is False:
         print(f"Invalid source path provided, {data['src_path']} could not be found.")
