@@ -1,3 +1,7 @@
+# pylint: disable=too-many-lines
+# pylint: disable=too-many-branches
+# pylint: disable=line-too-long
+# pylint: disable=bare-except
 """
     Contains the general methods for manipulating files.
 """
@@ -5,40 +9,32 @@
 # import json
 # import shutil
 from datetime import timezone
-import time
+# import time
 from datetime import datetime
 import gzip
-import json
+# import json
 import os
-import re
+# import re
 import io
 import shutil
 import traceback
 # from pathlib import Path
 
+from threading import Thread
+import logging
+from secure_delete import secure_delete
+import colemen_string_utils as strUtils
+# import ftputil
+import utils.resources
 import utils.dir as dirUtils
 import utils.objectUtils as obj
 import utils.file_read as read
 import utils.file_write as write
 import utils.file_search as search
-import utils.string_utils as strUtils
-from secure_delete import secure_delete
-from threading import Thread
-import colemen_string_utils as strUtils
-import logging
-import ftputil
-import utils.resources
+# import utils.string_utils as strUtils
 
 logger = logging.getLogger(__name__)
 
-# pylint: disable=line-too-long
-# DEST_PATH_SYNONYMS = utils.resources.DEST_PATH_SYNONYMS
-# SRC_PATH_SYNONYMS = utils.resources.SRC_PATH_SYNONYMS
-
-# todo - DOCUMENTATION FOR METHODS
-
-# // TODO:Get optional file access,create,modified formatting.
-
 
 def decompress(file_obj, compression="gzip"):
     '''
@@ -82,6 +78,24 @@ def decompress(file_obj, compression="gzip"):
 
 
 def compress(file_obj):
+    '''
+        gzip Compress the file provided.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `file_obj` {str|list|dict}
+            It can be a single path, list of paths, list of file objects, or a single file object.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 12-19-2021 14:00:20
+        `memberOf`: file
+        `version`: 1.0
+        `method_name`: compress
+    '''
     file_list = _gen_path_list(file_obj)
     if len(file_list) > 0:
         for file_path in file_list:
@@ -92,8 +106,8 @@ def _decompress_single_file_gzip(file_path):
     temp_path = strUtils.format.file_path(f"{os.path.dirname(file_path)}/{get_name_no_ext(file_path)}.decomp")
     contents = False
     try:
-        with gzip.open(file_path, 'rb') as ip:
-            with io.TextIOWrapper(ip, encoding='utf-8') as decoder:
+        with gzip.open(file_path, 'rb') as file:
+            with io.TextIOWrapper(file, encoding='utf-8') as decoder:
                 contents = decoder.read()
                 write.write(temp_path, contents)
     except gzip.BadGzipFile:
@@ -272,6 +286,31 @@ def _gen_path_list(file_obj):
 
 
 def import_project_settings(file_name):
+    '''
+        Used to import a json file, same as just reading a json file.
+        It only searches the working directory for a json file with a matching name.
+        Kept for backward compatibility.
+
+        ----------
+
+        Arguments
+        -------------------------
+        `file_name` {str}
+            The name of the settings file to search for and import.
+
+
+        Return {mixed}
+        ----------------------
+        The json decoded file.
+
+        Meta
+        ----------
+        `author`: Colemen Atwood
+        `created`: 12-19-2021 14:02:53
+        `memberOf`: file
+        `version`: 1.0
+        `method_name`: import_project_settings
+    '''
     settings_path = file_name
     if exists(settings_path) is False:
         settings_path = search.by_name(file_name, os.getcwd(), exact_match=False)
@@ -290,18 +329,18 @@ def _parse_copy_data_from_obj(file_obj):
             data['src_path'] = file_obj[0]
             data['dst_path'] = file_obj[1]
         else:
-            print(f"Invalid list/tuple provided for copy file. Must be [source_file_path, destination_file_path]")
+            print("Invalid list/tuple provided for copy file. Must be [source_file_path, destination_file_path]")
     if isinstance(file_obj, (dict)):
         for syn in utils.resources.SRC_PATH_SYNONYMS:
-            synvar = obj._gen_variations(syn)
-            for sv in synvar:
-                if sv in file_obj:
-                    data['src_path'] = file_obj[sv]
+            synvar = obj.gen_variations(syn)
+            for synonym_variant in synvar:
+                if synonym_variant in file_obj:
+                    data['src_path'] = file_obj[synonym_variant]
         for syn in utils.resources.DEST_PATH_SYNONYMS:
-            synvar = obj._gen_variations(syn)
-            for sv in synvar:
-                if sv in file_obj:
-                    data['dst_path'] = file_obj[sv]
+            synvar = obj.gen_variations(syn)
+            for synonym_variant in synvar:
+                if synonym_variant in file_obj:
+                    data['dst_path'] = file_obj[synonym_variant]
 
     if exists(data['src_path']) is False:
         print(f"Invalid source path provided, {data['src_path']} could not be found.")
@@ -424,9 +463,9 @@ def copy(src, dest=False, **kwargs):
                     if copy_obj['src_path'] is not None and copy_obj['dst_path'] is not None:
                         copy_list.append(_parse_copy_data_from_obj(item))
             if isinstance(src, (dict)):
-                copy_obj = _parse_copy_data_from_obj(item)
+                copy_obj = _parse_copy_data_from_obj(src)
                 if copy_obj['src_path'] is not None and copy_obj['dst_path'] is not None:
-                    copy_list.append(_parse_copy_data_from_obj(item))
+                    copy_list.append(_parse_copy_data_from_obj(src))
     else:
         copy_obj = _parse_copy_data_from_obj([src, dest])
         copy_list.append(copy_obj)
@@ -444,7 +483,7 @@ def _copy_files_from_array(file_list, ftp=None, attempted_delete=False):
     for file in file_list:
         try:
             if ftp is not None:
-                dirUtils.create(os.path.dirname(file['dst_path']), False, ftp)
+                dirUtils.create(os.path.dirname(file['dst_path']), False, ftp=ftp)
                 ftp.upload_if_newer(file['src_path'], file['dst_path'])
                 continue
 
@@ -655,10 +694,10 @@ def get_data(file_path, **kwargs):
 
             return file_data
         except FileNotFoundError as error:
-            logger.warning(f"Error: {error}")
+            logger.warning("Error: %s", error)
             return None
     else:
-        logger.warning(f"Failed to find the file: {file_path}")
+        logger.warning("Failed to find the file: %s", file_path)
 
 
 def get_modified_time(file_path, ftp=None):
@@ -807,7 +846,7 @@ def gen_relative_path(src_base, dst_base, file_path):
     '''
         Finds the relative path from the src_base, dst_base and file path.
 
-        example: 
+        example:
             file_path = r"Z:\\Structure\\Ra9\\2021\\21-0134_EquariServer\\equari_php\\pcom\\animal\\read\\PCOM_getOMNIAnimalData.php"\n
             src_base = r"Z:\\Structure\\Ra9\\2021\\21-0134_EquariServer\\equari_php"\n
             dst_base = r"/equari_php/\n
@@ -895,7 +934,7 @@ def gen_dst_path(src_base, dst_base, file_path):
         Generates the destination path for a file.
 
 
-        example: 
+        example:
             file_path = r"Z:\\Structure\\Ra9\\2021\\21-0134_EquariServer\\equari_php\\pcom\\animal\\read\\PCOM_getOMNIAnimalData.php"\n
             src_base = r"Z:\\Structure\\Ra9\\2021\\21-0134_EquariServer\\equari_php"\n
             dst_base = r"/equari_php/\n
@@ -940,7 +979,7 @@ def gen_src_path(src_base, dst_base, file_path):
         Generates the source path for a file.
 
 
-        example: 
+        example:
             src_base = r"Z:\\Structure\\Ra9\\2021\\21-0134_EquariServer\\equari_php"\n
             dst_base = r"/equari_php/"\n
             file_path = r"\\equari_php\\pcom\\animal\\read\\PCOM_getOMNIAnimalData.php"\n
